﻿using System;
using MccDaq;
using System.IO;
using System.Linq;

namespace Ain_Aout_Continous
{
    class Program
    {
        public const int iBLOCKSIZE = 4096;     //input block size
        public const int oBLOCKSIZE = 2048;     //output block size

        public const int iFREQ = 5000;          //input sample rate
        public const int oFREQ = 50000;         //output update rate

        public const int FIRSTCHANNEL = 0;
        public const int LASTCHANNEL = 1;

        public const int iCHANCOUNT = LASTCHANNEL - FIRSTCHANNEL +1;        //input channel count
        public const int oCHANCOUNT = LASTCHANNEL - FIRSTCHANNEL + 1;       //output channel count
        
        public const int iBUFFERSIZE = iBLOCKSIZE * iCHANCOUNT; //input buffer size
        public const int oBUFFERSIZE = oBLOCKSIZE * oCHANCOUNT; //output buffer size

        public const int iHALFBUFFSIZE = iBUFFERSIZE / 2;
        public const int oHALFBUFFSIZE = oBUFFERSIZE / 2;
        
        //The following devices have been tried...Uncomment yours
        //public const string DEVICE = "PCI-DAS6052"; public const int RESOLUTION = 65535; public const ScanOptions ExtraOpts = ScanOptions.BurstMode | ScanOptions.BlockIo;
        //public const string DEVICE = "USB-1608GX-2AO"; public const int RESOLUTION = 65535; public const ScanOptions ExtraOpts = ScanOptions.BurstMode | ScanOptions.BlockIo;
        //public const string DEVICE = "USB-1616HS-4"; public const int RESOLUTION = 65535; public const ScanOptions ExtraOpts = ScanOptions.BlockIo;
        //public const string DEVICE = "USB-1616HS-2"; public const int RESOLUTION = 65535; public const ScanOptions ExtraOpts = ScanOptions.BlockIo;
        //public const string DEVICE = "USB-2527"; public const int RESOLUTION = 65535; public const ScanOptions ExtraOpts = ScanOptions.BlockIo;
        //public const string DEVICE = "USB-1208HS-4AO"; public const int RESOLUTION = 4096; public const ScanOptions ExtraOpts = ScanOptions.BurstMode|ScanOptions.BlockIo;
        public const string DEVICE = "USB-1408FS-Plus"; public const int RESOLUTION = 4096; public const ScanOptions ExtraOpts = ScanOptions.BlockIo;
        //public const string DEVICE = "USB-1208FS-Plus"; public const int RESOLUTION = 4096; public const ScanOptions ExtraOpts = ScanOptions.BlockIo;

        public static StreamWriter fStream;
        public static IntPtr OutBuf;
        public static IntPtr buffer;
        public static bool inLower;
        public static bool outLower;
        public static ushort[] inArray;
        public static ushort[] outArray;
     
        public static MccBoard daq;

        public static MccDaq.ErrorInfo RetVal;
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************

        static void Main(string[] args)
        {

            MccDaq.ErrorInfo RetVal;
            int BoardNum = 0;
            int iRate = iFREQ; //input rate
            int oRate = oFREQ; //output rate
            inLower = true;
            outLower = true;
            Console.WriteLine("Connecting to {0}",DEVICE);
            BoardNum = GetBoardNum(DEVICE);

            if (BoardNum == -1)
            {
                Console.WriteLine("No {0} detected!", DEVICE);
                WaitForKey();
                return;
            }
            else
            { ///////// BEGIN PROGRAM ///////////
                daq = new MccDaq.MccBoard(BoardNum);

                //create the driver buffers
                OutBuf = MccService.WinBufAllocEx(oBUFFERSIZE);
                buffer = MccService.ScaledWinBufAllocEx(iBUFFERSIZE);
                Console.WriteLine("{0} Connected",DEVICE);

                if ((OutBuf == IntPtr.Zero) || (buffer == IntPtr.Zero))
                {
                    Console.WriteLine("Insufficient Memory");
                    WaitForKey();
                    return;
                }

                
                //create the user output buffer.
                outArray = new ushort[oBUFFERSIZE];
                //create the waveforms
                for(int i=0;i<oCHANCOUNT;i++)
                    GetWaveForm(3, i, outArray); //start with sine wave

                //write the user data to the driver buffer
                MccService.WinArrayToBuf(outArray, OutBuf, 0, oBUFFERSIZE);
                //start the waveforms 
                RetVal = daq.AOutScan(  FIRSTCHANNEL,
                                        LASTCHANNEL,
                                        oBUFFERSIZE,
                                        ref oRate,
                                        Range.Bip10Volts,
                                        OutBuf,
                                        ScanOptions.Background | ScanOptions.Continuous
                                      );
                if (IsError("AOutScan",RetVal) == 1) return;


                short[] chArray = new short[iCHANCOUNT]; //configuration array for channel numbers
                for(short i=0; i<iCHANCOUNT;i++)
                    chArray[0] = i;


 

                fStream = new StreamWriter(@"C:\Users\Public\Documents\DataFile.asc");
                CreateFileHeaders(chArray); //writes basic info to the beginning of the file
                System.ConsoleKeyInfo cki;
                WaitForKey();
 
                //begin analog input
                RetVal = daq.AInScan(FIRSTCHANNEL,
                                        LASTCHANNEL,
                                        iBUFFERSIZE,
                                        ref iRate,
                                        Range.Bip10Volts,
                                        buffer,
                                        ScanOptions.Background | ScanOptions.Continuous | ExtraOpts
                                    );
                if (IsError("AInScan", RetVal) == 1) return; bool exit = false;
                
                //do loop to switch waveforms and read/write data
                do
                {
                    if (Console.KeyAvailable)
                    {
                        cki = Console.ReadKey(true);
                        switch (cki.Key)
                        {
                            case ConsoleKey.D1: 
                            case ConsoleKey.NumPad1:
                                if ((cki.Modifiers & ConsoleModifiers.Alt) != 0)
                                    GetWaveForm(1, 1, outArray);
                                else
                                    GetWaveForm(1,0,outArray);

                                break;
                            case ConsoleKey.D2:
                            case ConsoleKey.NumPad2:
                                if ((cki.Modifiers & ConsoleModifiers.Alt) != 0)
                                    GetWaveForm(2, 1, outArray);
                                else
                                    GetWaveForm(2, 0, outArray);
                                break;
                            case ConsoleKey.D3:
                            case ConsoleKey.NumPad3:
                                if ((cki.Modifiers & ConsoleModifiers.Alt) != 0)
                                    GetWaveForm(3, 1, outArray);
                                else
                                    GetWaveForm(3, 0, outArray);
                                break;
                            case ConsoleKey.D4:
                            case ConsoleKey.NumPad4:
                                if ((cki.Modifiers & ConsoleModifiers.Alt) != 0)
                                    GetWaveForm(4, 1, outArray);
                                else
                                    GetWaveForm(4, 0, outArray);
                                break;
                            case ConsoleKey.D5:
                            case ConsoleKey.NumPad5:
                                if ((cki.Modifiers & ConsoleModifiers.Alt) != 0)
                                    GetWaveForm(5, 1, outArray);
                                else
                                    GetWaveForm(5, 0, outArray);
                                break;
                            case ConsoleKey.D6:
                            case ConsoleKey.NumPad6:
                                if ((cki.Modifiers & ConsoleModifiers.Alt) != 0)
                                    GetWaveForm(6, 1, outArray);
                                else
                                    GetWaveForm(6, 0, outArray);
                                break;
                            case ConsoleKey.Spacebar:
                                exit = true;
                                break;

                        }
                    }
                    if(ReadInputBuffer()==1) return;
                    if(WriteOutputBuffer()==1) return;

                   
                } while (!exit);

                //flush buffered data out to disk
                fStream.Flush();
                //close file
                fStream.Close();

                //stop the  acquisition
                RetVal = daq.StopBackground(FunctionType.AiFunction);
                RetVal = daq.StopBackground(FunctionType.AoFunction);

                //free up memory
                MccService.WinBufFreeEx(buffer);
                MccService.WinBufFreeEx(OutBuf);

                return;
            }
        }
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************

        public static int ReadInputBuffer()
        {
            int Count = 0;
            int Index = 0;
            short daqStatus;
            inArray = new ushort[iBUFFERSIZE];

            RetVal = daq.GetStatus(out daqStatus, out Count, out Index, FunctionType.AiFunction);
            if ((Index > iHALFBUFFSIZE) & inLower) //check for 50% more data
            {
                //get lower half of buffer of the buffer
                RetVal = MccService.WinBufToArray(buffer, inArray, 0, iHALFBUFFSIZE);

                IsError("ScaledWinBufToArray", RetVal);

                DisplayData(inArray, iHALFBUFFSIZE / iCHANCOUNT);
                inLower = false; //flag that controls the next read
            }
            else if ((Index < iHALFBUFFSIZE) & !inLower)
            {
                //get the upper half of the buffer
                RetVal = MccService.WinBufToArray(buffer, inArray, iHALFBUFFSIZE, iHALFBUFFSIZE);

                IsError("ScaledWinBufToArray",RetVal);

                DisplayData(inArray, iHALFBUFFSIZE / iCHANCOUNT);
                inLower = true;//flag that controls the next read
            }

            return 0;
        }
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************

        public static int WriteOutputBuffer()
        {
            int Count = 0;
            int Index = 0;
            short daqStatus = 0;
            ushort[] temp = new ushort[oHALFBUFFSIZE];
            
            RetVal = daq.GetStatus(out daqStatus, out Count, out Index, FunctionType.AoFunction);
            //wait for the index to go past the half way point befor updating 
            //the bottom half of the buffer
            if ((Index > oHALFBUFFSIZE) & outLower)
            {
                //get the lower half of the buffer
                temp = outArray.Take(oHALFBUFFSIZE).ToArray();
                MccService.WinArrayToBuf(temp, OutBuf, 0, oHALFBUFFSIZE);
 
                if (IsError("WinArrayToBuf", RetVal) == 1) return 1;

                outLower = false; //flag that controls the next write
            }
            //wait for the index to roll over to the bottom half before updating 
            //the upper half of the buffer
            else if ((Index <= oHALFBUFFSIZE) & !outLower)
            {
                //get the upper half of the buffer array
                temp = outArray.Skip(oHALFBUFFSIZE).Take(oHALFBUFFSIZE).ToArray();
                MccService.WinArrayToBuf(temp, OutBuf, oHALFBUFFSIZE, oHALFBUFFSIZE);
                if(IsError("WinArrayToBuf", RetVal)==1) return 1;

                outLower = true; //flag that controls the next write
            }
            return 0;
        }

        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************
        public static int IsError(string s, ErrorInfo e)
        {//check for error function
            if (e.Value != 0)
            {
                Console.WriteLine("Function {0}: {1}",s,e.Message);
                WaitForKey();
                return 1;
            }
            return 0;
        }
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************
        public static int GetWaveForm(ushort typ, int ch, ushort[] p)
        { //receive buffer pointer the create a waveform type for a designated channel
            ushort tot = 0;
            ushort dif = 0;
            int idx = ch;
            switch (typ)
            {
                case 1://zero wave

                    for (int i = 0; i < oBLOCKSIZE; i++)
                    {
                        p[idx] = RESOLUTION/2;
                        idx += oCHANCOUNT;

                    }
                    return 0;

                case 2://ramp wave 
                    tot = 0;
                    dif = (ushort)(RESOLUTION / oBLOCKSIZE);
                    for (int i = 0; i < oBLOCKSIZE; i++)
                    {                       
                        tot += dif;
                        p[idx] = tot;
                        idx += oCHANCOUNT;
                    }
                    return 0;

                case 3://sine wave
                    
                    double angle = 6.28318 / oBLOCKSIZE;
                    double value;
                    for (int i = 0; i < oBLOCKSIZE; i++)
                    {
                        value = (System.Math.Sin(angle * i) + 1) * (RESOLUTION/2);
                        p[idx] = System.Convert.ToUInt16(value);
                        idx += oCHANCOUNT;                  
                    }
                    return 0;
 
                case 4: //square wave
                    for (int i = 0; i < (oBLOCKSIZE / 2); i++)
                    {
                        p[idx] = RESOLUTION/2;
                        idx += oCHANCOUNT;
                    }
                    for (int i = (oBLOCKSIZE / 2); i < oBLOCKSIZE; i++)
                    {
                        p[idx] = RESOLUTION;
                        idx += oCHANCOUNT;
                    }
                    return 0;

                case 5: //rectangle
                    for (int i = 0; i < (oBLOCKSIZE / 2); i++)
                    {
                        p[idx] = 0;
                        idx += oCHANCOUNT;
                    }
                    for (int i = (oBLOCKSIZE / 2); i < oBLOCKSIZE; i++)
                    {
                        p[idx] = RESOLUTION;
                        idx += oCHANCOUNT;
                    }
                    return 0;

                case 6: //triangle
                    tot = 0;
                    dif = (ushort)(RESOLUTION / (oBLOCKSIZE / 2));
                    for (int i = 0; i < (oBLOCKSIZE / 2); i++)
                    {
                        tot += dif;
                        p[idx] = tot;
                        idx += oCHANCOUNT;
                    }
                    for (int i = (oBLOCKSIZE / 2); i < oBLOCKSIZE; i++)
                    {
                        tot -= dif;
                        p[idx] = tot;
                        idx += oCHANCOUNT;
                    }
                    return 0;
            }
            return 1;
        }
 
 
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************

        public static int GetBoardNum(string dev)
        {
            for (int BoardNum = 0; BoardNum < 99; BoardNum++)
            {
                MccDaq.MccBoard daq = new MccDaq.MccBoard(BoardNum);
                if (daq.BoardName.Contains(dev))
                {
                    Console.WriteLine("{0} board number = {1}\n", dev, BoardNum.ToString());
                    daq.FlashLED();
                    return BoardNum;
                }
            }
            return -1;
        }
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************

        public static void WaitForKey()
        {
            Console.WriteLine("\nPress <SPACE> to continue...");
         
            System.ConsoleKeyInfo cki;
            do
            {
                cki = Console.ReadKey(true);
            } while (cki.Key != ConsoleKey.Spacebar);
        }
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************

        public static void DisplayData(ushort[] datArray, int rows)
        {
            //Writes data to screen and to file
            int i = 0;
            for (int row = 0; row < rows; row++)
            {
 
                for (int c = 0; c < iCHANCOUNT; c++)
                {
                    Console.Write("{0}\t", System.Convert.ToUInt16(datArray[i]).ToString("000000").PadLeft(10));
                    fStream.Write("{0}\t", System.Convert.ToUInt16(datArray[i]).ToString("000000"));
                    i++;
                }
                Console.Write("\r\n");
                fStream.Write("\r\n");
            }
            fStream.Flush();
        }
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************

        public static void CreateFileHeaders(short[] x)
        { // creates file header for both the data file and the screen output
            string s = "This program outputs a waveform on AOut 0 & 1\n";
            s += "Number keys 1-6 select waveforms and the <ALT> key swithes between channels\n";
            Console.WriteLine("INSTRUCTIONS");
            Console.WriteLine("{0}",s);
 

            //''''''''''''''''''''' create text file header strings ''''''''''''''''
            string[] hdr = new string[5];
            hdr[0] = "Recording date     : {0}";
            hdr[1] = "Block length       : {0}";
            hdr[2] = "Delta              : {0} sec.";
            hdr[3] = "Number of channels : {0}";

            //''''''''''''''''''''''' build column headers string '''''''''''''''''''

            for (int j = 0; j < iCHANCOUNT; j++)
            {
                hdr[4] = hdr[4] + String.Format("In{0} [V]", x[j]);
                hdr[4] = hdr[4] + "\t";

            }
            hdr[4].Trim('\t');

            //'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
            DateTime MyDate = DateTime.Now;

            fStream.WriteLine(hdr[0], MyDate);
            fStream.WriteLine(hdr[1], iHALFBUFFSIZE / iCHANCOUNT);
            fStream.WriteLine(hdr[2], string.Format("{0:0.00000}",(float)(1.0f / iFREQ)));
            fStream.WriteLine(hdr[3], iCHANCOUNT);
            fStream.WriteLine(hdr[4]);

            Console.WriteLine(hdr[0], MyDate);
            Console.WriteLine(hdr[1], iHALFBUFFSIZE / iCHANCOUNT);
            Console.WriteLine(hdr[2], string.Format("{0:0.00000}", (float)(1.0f / iFREQ)));
            Console.WriteLine(hdr[3], iCHANCOUNT);
            Console.WriteLine(hdr[4]);

        }
        // *********************************************************************************************************
        // *********************************************************************************************************
        // *********************************************************************************************************
    }
}
